#!/bin/sh
#
# Protected by US patent 6,006,034; patents pending.
# Copyright 2022 Flexera Software LLC.
# Reproduction, adaptation, or translation without prior permission
# is prohibited.
#
#
# Set MGSFT_RUN_CONFIGURE to control running of the configuration wizard.
#   Never run: "no", run if no other configuration: "yes", always run: "force"
# Optional argument:
#   Data directory: Path for storing files created by the agent.
#       Default: /var/opt/managesoft
# 

COMMONAPPDATAFOLDER=`echo "$1" | sed 's/\/*$//'`
BOOTSTRAP_TYPE=none

# Determine installation directory based on the path of this shell script.
CONFIGURE_SCRIPT_PATH="`dirname "$0"`"
INSTALLDIR="`cd "${CONFIGURE_SCRIPT_PATH}/.." && pwd`"

if [ -z "$INSTALLDIR" ]
then
	echo "ERROR:   The configure script could not determine the installation directory"
	echo "         of the agent.  Please retry the installation of the agent package."
	exit 1
fi


if [ -z "$COMMONAPPDATAFOLDER" ]
then
	COMMONAPPDATAFOLDER=/var/opt/managesoft
fi

echo "Configuring ManageSoft for Managed Devices ..."
echo "Installation directory: \"$INSTALLDIR\""
echo "Data directory: \"$COMMONAPPDATAFOLDER\""

#-----------------------------
# Create the main configure file
#-----------------------------
if printf "# ManageSoft Managed Device Main Configuration
[ManageSoft]
InstallDir=$INSTALLDIR
CommonAppDataFolder=$COMMONAPPDATAFOLDER
ETCPVersion=18.0.0
ETCPPublisher=Flexera
PackageType=dpkg
" > /etc/managesoft.ini.tmp && mv /etc/managesoft.ini.tmp /etc/managesoft.ini
then
    echo "Created /etc/managesoft.ini"
    chmod 644 /etc/managesoft.ini
else
    echo "Failed to create /etc/managesoft.ini"
    rm -f /etc/managesoft.ini.tmp
    exit 1
fi

#-----------------------------
# Create data directory tree
#-----------------------------
mkdir -p "$COMMONAPPDATAFOLDER" \
	&& chmod 700 "$COMMONAPPDATAFOLDER"

mkdir -p "$COMMONAPPDATAFOLDER/etc" \
	&& chmod 700 "$COMMONAPPDATAFOLDER/etc"


#-----------------------------
# Set default configuration
#-----------------------------
if [ ! -s "$COMMONAPPDATAFOLDER/etc/config.ini" ]
then
	printf "Creating default configuration: "
	cp "$INSTALLDIR/install/default.ini" "$COMMONAPPDATAFOLDER/etc/config.ini" \
		&& chmod 600 "$COMMONAPPDATAFOLDER/etc/config.ini"
	echo "Done"
else
	# Make sure settings missing from config.ini have default values
	printf "Updating configuration with defaults: "
	chmod 600 "$COMMONAPPDATAFOLDER/etc/config.ini"
	"$INSTALLDIR/bin/mgsconfig" -n -i "$INSTALLDIR/install/default.ini"
	echo "Done"
fi

#-----------------------------
# Configure managesoft from saved "$COMMONAPPDATAFOLDER/etc/config.save.ini"
#-----------------------------
for MGSFT_CONFIG_INI in \
	"$COMMONAPPDATAFOLDER/etc/config.save.ini"
do
	if [ -s "$MGSFT_CONFIG_INI" ]
	then
		printf "Applying saved configuration \"$MGSFT_CONFIG_INI\": "
		"$INSTALLDIR/bin/mgsconfig" -i "$MGSFT_CONFIG_INI"
		BOOTSTRAP_TYPE=old
		echo "Done"
	fi

	rm -f "$MGSFT_CONFIG_INI"
done

#-----------------------------
# Check if we are performing an upgrade
#-----------------------------
if [ -f "/var/tmp/mgsft_config.upgrade" ]
then
	BOOTSTRAP_TYPE=upgrade
	rm -f "/var/tmp/mgsft_config.upgrade"
fi

#-----------------------------
# Configure using answer file
#-----------------------------
MGSFT_PARAM_FILE=/var/tmp/mgsft_rollout_response
if [ -s "$MGSFT_PARAM_FILE" ]
then
	echo "Applying bootstrap answer file \"$MGSFT_PARAM_FILE\" ..."
	"$INSTALLDIR/bin/mgsconfig" -a "$MGSFT_PARAM_FILE"
	BOOTSTRAP_TYPE=answer
fi

rm -f "$MGSFT_PARAM_FILE"

#-----------------------------
# Configure certificate
#-----------------------------
MGSFT_CERT_FILE=/var/tmp/mgsft_rollout_cert
if [ -s "$MGSFT_CERT_FILE" ]
then
	echo "Installing bootstrap certificate file \"$MGSFT_CERT_FILE\" ..."
	mkdir -p "$COMMONAPPDATAFOLDER/etc/ssl"
	cp "$MGSFT_CERT_FILE" "$COMMONAPPDATAFOLDER/etc/ssl/cert.pem"
fi

rm -f ${MGSFT_CERT_FILE}

#-----------------------------
# Configure client certificate and private key
#-----------------------------
MGSFT_CLIENT_CERT_FILE=/var/tmp/mgsft_rollout_client_cert
if [ -s "$MGSFT_CLIENT_CERT_FILE" ]
then
	echo "Installing bootstrap client certificate file \"$MGSFT_CLIENT_CERT_FILE\" ..."
	mkdir -p "$COMMONAPPDATAFOLDER/etc/ssl/client"
	cp "$MGSFT_CLIENT_CERT_FILE" "$COMMONAPPDATAFOLDER/etc/ssl/client/client_cert.pem"
fi
rm -f ${MGSFT_CLIENT_CERT_FILE}

MGSFT_CLIENT_PRIVATE_KEY_FILE=/var/tmp/mgsft_rollout_client_private_key
if [ -s "$MGSFT_CLIENT_PRIVATE_KEY_FILE" ]
then
	echo "Installing bootstrap client private key file \"$MGSFT_CLIENT_PRIVATE_KEY_FILE\" ..."
	mkdir -p "$COMMONAPPDATAFOLDER/etc/ssl/client/private"
	cp "$MGSFT_CLIENT_PRIVATE_KEY_FILE" "$COMMONAPPDATAFOLDER/etc/ssl/client/private/client_key.pem"
fi

rm -f ${MGSFT_CLIENT_PRIVATE_KEY_FILE}

#-----------------------------
# Configuration using manual wizard
#-----------------------------
if [ "$BOOTSTRAP_TYPE" = "none" -a "$MGSFT_RUN_CONFIGURE" = "yes" \
	-o "$MGSFT_RUN_CONFIGURE" = "force" ]
then
	"$INSTALLDIR/bin/managesoft-configure"
	BOOTSTRAP_TYPE=manual
fi

#-----------------------------
# Add and start services
#-----------------------------
# Install ManageSoft services for Linux
if [ -x /usr/bin/systemctl ]
then
	printf "Reloading the systemd manager configuration: "
	/usr/bin/systemctl daemon-reload > /dev/null
	echo "done"
fi

if [ -x /usr/sbin/update-rc.d ]
then
	printf "Adding service startup configuration with update-rc.d: "
	/usr/sbin/update-rc.d ndtask start 99 2 3 4 5 . stop 01 0 1 6 . > /dev/null
	/usr/sbin/update-rc.d mgsusageag start 99 2 3 4 5 . stop 01 0 1 6 . > /dev/null
	echo "done"
elif [ -x /sbin/insserv ]
then
	printf "Adding service startup configuration with insserv: "
	/sbin/insserv ndtask
	/sbin/insserv mgsusageag
	echo "done"
elif [ -x /sbin/chkconfig ]
then
	printf "Adding service startup configuration with chkconfig: "
	/sbin/chkconfig --add ndtask
	/sbin/chkconfig --add mgsusageag
	echo "done"
else
	printf "Adding service startup configuration with symlinks: "
	if [ -d /etc/rc2.d ]
	then
		# Red Hat Linux style bootstrapping
		RCDIR=/etc
		RELINITD=../init.d
	elif [ -d /etc/init.d/rc2.d ]
	then
		# SuSE Linux style bootstrapping
		RCDIR=/etc/init.d
		RELINITD=..
	fi

	if [ -d $RCDIR ]
	then
		for runlevel in rc1.d/K01 rc2.d/S99 rc3.d/S99 rc4.d/S99 rc5.d/S99 rc6.d/K01
		do
			ln -sf $RELINITD/ndtask $RCDIR/${runlevel}ndtask
			ln -sf $RELINITD/mgsusageag $RCDIR/${runlevel}mgsusageag
		done
		echo "done"
	else
		echo "Failed to find path"
	fi
fi

# Start ManageSoft services for GNU/Linux
echo "Starting ManageSoft services..."
/etc/init.d/ndtask start; /etc/init.d/mgsusageag start


#-----------------------------
# Import existing schedule or set heartbeat schedule
#-----------------------------
HAVE_SCHEDULE=no

# Find an .nds schedule file
NDSFILE="`find "$COMMONAPPDATAFOLDER/scheduler" /var/opt/managesoft/scheduler /var/opt/managesoft/live/Schedules /var/lib/managesoft/live/Schedules -name \*.nds -type f 2>/dev/null | head -1`"
if [ -s "$NDSFILE" ]
then
	printf "Importing schedule \"%s\": " "$NDSFILE"
	if "$INSTALLDIR/bin/ndschedag" -o DisableTrigger=Now "$NDSFILE"
	then
		HAVE_SCHEDULE=yes
		echo "Done"
	else
		echo "Failed"
	fi
fi

if [ "$HAVE_SCHEDULE" = "no" ]
then
	printf "Setting heartbeat schedule: "
	# Create a temporary schedule and register it with ndschedag
# This heartbeat schedule will run once a day at the same random time
TMPNDSFILE=/tmp/heartbeat_$$.nds
rm -f "$TMPNDSFILE" && cat << EOS > "$TMPNDSFILE" && "$INSTALLDIR/bin/ndschedag" -o DisableTrigger=Now "$TMPNDSFILE"
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE Schedule PUBLIC "ManageSoft schedule" "http://www.managesoft.com/dtd/schedule.dtd">

<Schedule NAME="Daily Heartbeat Schedule" TIMEDATESTAMP="19800101T000000Z" SCHEDSCHEMA="60">
	<Event ID="{00000000-007a-00a5-07cf-012a3a4e50f7}" NAME="Daily Heartbeat" NETWORK="False" CATCHUP="Never">
		<LogicalCommand ACTION="Update" COMPONENT="Scheduler"/>
		<Trigger TYPE_PARAM="1" TYPE="Daily" DATESTART="19800101" TIMESTART="000000" TIMEWINDOW="86400"/>
	</Event>
</Schedule>
EOS
rm -f "$TMPNDSFILE"

	echo "Done"
fi

#-----------------------------
# Display bootstrap status
#-----------------------------
if [ "$BOOTSTRAP_TYPE" = "none" ]
then
	echo "*****************************************************************************"
	echo "**  NOTE:    ManageSoft was installed without a configuration."
	echo "**           Run \"$INSTALLDIR/bin/managesoft-configure\" to "
	echo "**           configure ManageSoft."
	echo "*****************************************************************************"

	# Run ndtrack for hardware inventory, as mgsconfig was not run to have this done for us. Include the INSTALLER_SKIP_NDTRACK_HW_SCAN env var bypass for FIAT
	if [ -z "$INSTALLER_SKIP_NDTRACK_HW_SCAN" ]
	then
		nohup $INSTALLDIR/bin/ndtrack -o WMI=true -o Hardware=true -o ManageSoftPackages=false -o MSI=false -o PlatformSpecificPackages=false -o Software=false -o TrackProductKey=false -o IncludeRegistryKey= -o IncludeDirectory= -o EmbedFileContentDirectory= -o PerformSymantecSFScan=false -o PerformIBMWebSphereMQScan=false -o InventorySettingsPath= -o LogFile=/var/opt/managesoft/log/cfg-tracker.log < /dev/null > /dev/null 2>&1 &
	fi

elif [ "$BOOTSTRAP_TYPE" = "old" ]
then
	echo "*****************************************************************************"
	echo "**  NOTE:    ManageSoft is using the configuration from a previous install."
	echo "**           You may need to run \"$INSTALLDIR/bin/managesoft-configure\""
	echo "**           to update the ManageSoft configuration."
	echo "*****************************************************************************"
elif [ "$BOOTSTRAP_TYPE" = "upgrade" ]
then
	echo "ManageSoft was configured as part of an upgrade."
elif [ "$BOOTSTRAP_TYPE" = "answer" ]
then
	echo "ManageSoft was configured using an answer file."
elif [ "$BOOTSTRAP_TYPE" = "manual" ]
then
	echo "ManageSoft was manually configured."
fi

exit 0
